package main


import (
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"img/app/config"
	"img/app/lib/logger"
	"img/app/router"
)

// @Title 程序入口
func main() {
	// 初始化项目
	closes := initApp()
	// 关闭资源
	for _, fn := range closes {
		defer fn()
	}

	//// 模式处理
	if config.Config.Debug {
		gin.ForceConsoleColor()
	} else {
		gin.SetMode(gin.ReleaseMode)
	}

	// 路由
	engine := gin.Default()

	// 设置路由
	router.SetRouter(engine)

	// 运行
	if err := engine.Run(config.Config.Server.Addr); err != nil {
		logger.Logger.Fatal("运行错误", zap.Any("err", err))
		return
	}
}
